/* wordsort.c  */
/* Part of a bilingual dictionary program.                        */
/* Loads an array of words, and sorts them based on               */
/*  their sort forms.                                             */

/* Author: Zach Tomaszewski                                       */
/* Date: 27 Feb 2003                                              */

#include "dictionary.h"

Word wordList[MAXWORDCOUNT];
int wordListCount = 0;


/*
 Takes a string of max length LINE, which must contain a tab.
 Splits the line on the tab, putting the first half in Word.english
   and the second half in Word.japanese.
   Copies Word.english to Word.sortform and removes any leading
   parethetical modifiers.
   Passes the new Word along to loadWord. 
 If there is no tab, does nothing.
 Returns 1 on success, 0 on failure.
*/
int loadLine(char *line) {
  char *tab;
  Word newWord;
  tab = strchr(line, '\t');
  if (tab != NULL) {
    *tab = '\0';  // set to null
    strcpy(newWord.english, line);
    strcpy(newWord.sortForm, line);
    strcpy(newWord.japanese, tab+1);
    toSortForm(newWord.sortForm);
    loadWord(newWord);
    return(1);
  }else{
    return(0);
  }
}


/*
 Requires a pointer to a string.
 Edits that string in place by removing any preceding parethetical
  modifiers and converting to lowercase.
*/
void toSortForm(char *sortform){
  char *cut;
  int i = 0;
  while (sortform[i] != '\0'){
    sortform[i] = (char) tolower(sortform[i]);
    i++;
  }
  if (sortform[0] == '(') {
    cut = strpbrk(sortform, ")");
    cut++; //to space just past found char
    strcpy(sortform, cut);
    toSortForm(sortform);  //remove any more mods or spaces
  }else if (sortform[0] == '[') {
    cut = strpbrk(sortform, "]");
    cut++; //to space just past found char
    strcpy(sortform, cut);
    toSortForm(sortform);
  }else if (sortform[0] == ' ') {
    strcpy(sortform, sortform + 1); //skip starting space
    toSortForm(sortform);
  }
}


/*
 Takes a Word with all three fields set.
 Inserts the Word into wordList and increases wordListCount.
 Returns 0 if successful, or 1 if array is full.
*/
int loadWord(Word newWord){
  Word temp;
  int i = 0;
  if (wordListCount >= MAXWORDCOUNT) {
    return 1; //array full
  }
  while (i < wordListCount && 
         strcmp(newWord.sortForm, wordList[i].sortForm) >= 0){
    i++;
  } 
  if (i == wordListCount) { //at end of array, just add
    wordList[wordListCount++] = newWord;
  }else{
    do{
      temp = wordList[i];
      wordList[i] = newWord;
      newWord = temp;
      i++;
    }while (i <= wordListCount);
    wordList[++wordListCount]; //add last one to end
  }
}


/*
 Prints wordList to the screen.
  Formats it as Word.english first, at least one space, 
  and then Word.japanese.  One Word per line.
*/
void printWordList(){
  register int i = 0;
  register int j = 0;
  int englishLength;

  for (i=0; i < wordListCount; i++){
    printf(wordList[i].english);
    englishLength = strlen(wordList[i].english);
    for(j = 0; j <= (MAXWORD - englishLength + 1); j++){
      putchar(' ');
    }
    printf(wordList[i].japanese);
  }
}
